/*
 * Copyright (c) 2018,2021 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Mon Aug 20 2018
 */
#include "acftsens.h"

#include <Eigen/Core>
#include <Eigen/Dense>

#include "ftsens_c_types.h"
#include "codels.h"

void
fts_get_frame(double x, double y, double z,
              double yaw, double pitch, double roll,
              double frame[36])
{
  using namespace Eigen;

  const Matrix<double, 3, 1> p(x, y, z);
  const Quaternion<double> q =
    AngleAxisd(roll, Vector3d::UnitX())
    * AngleAxisd(pitch, Vector3d::UnitY())
    * AngleAxisd(yaw, Vector3d::UnitZ());

  Map< Matrix<double, 6, 6, RowMajor> > F(frame);

  F <<
    q.matrix(),                     Matrix<double, 3, 3>::Zero(),
    q.matrix().rowwise().cross(-p), q.matrix();
}

void
fts_get_wrench(const strain2_data *data, const double frame[36], double w[6])
{
  using namespace Eigen;

  Map< const Matrix<double, 6, 6, RowMajor> > F(frame);
  Map< const Array<float, 6, 1> > S(data->fullscale);
  Map< Matrix<double, 6, 1> > W(w);
  Array<double, 6, 1> D;

  D <<
    data->force[0], data->force[1], data->force[2],
    data->torque[0], data->torque[1], data->torque[2];

  W = F * (D * S.cast<double>()).matrix();
}

void
fts_get_adc(const strain2_data *data,
            const ftsens_ids_calibration_s *calibration, double adc[6])
{
  using namespace Eigen;

  Map< const Matrix<double, 6, 6, RowMajor> > G(calibration->G);
  Map< const Matrix<double, 6, 1> > T(calibration->b);
  Map< Matrix<double, 6, 1> > ADC(adc);
  Matrix<double, 6, 1> D;

  D <<
    data->force[0], data->force[1], data->force[2],
    data->torque[0], data->torque[1], data->torque[2];

  ADC = G.inverse() * D - T;
}
