/*
 * Copyright (c) 2021 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *					Anthony Mallet on Tue Jul 27 2021
 */
#ifndef H_FTSENS_CODELS
#define H_FTSENS_CODELS

#include "ftsens_c_types.h"

#include <aio.h>
#include <errno.h>
#include <inttypes.h>
#include <stdbool.h>
#include <string.h>

#include "icub/icub.h"

struct ftsens_hw {
  struct icub_dev *dev;
  struct icub_node *node;
};

struct ftsens_log_s {
  struct aiocb req;
  char buffer[4096];
  bool pending, skipped;
  uint32_t decimation;
  size_t missed, total;

# define ftsens_logfmt	"%g "
# define ftsens_log_header                                           \
  "ts "                                                              \
  "fx fy fz  tx ty tz"
# define ftsens_log_line                                             \
  "%" PRIu64 ".%09d "                                                \
  ftsens_logfmt ftsens_logfmt ftsens_logfmt                          \
  ftsens_logfmt ftsens_logfmt ftsens_logfmt
};

static inline genom_event
ftsens_e_sys_error(const char *s, genom_context self)
{
  ftsens_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  strerror_r(d.code, d.what + l, sizeof(d.what) - l);
  return ftsens_e_sys(&d, self);
}

#ifdef __cplusplus
extern "C" {
#endif

  void	fts_get_frame(double x, double y, double z,
                double yaw, double pitch, double roll,
                double frame[36]);
  void	fts_get_wrench(const strain2_data *data, const double frame[36],
                double w[6]);
  void	fts_get_adc(const strain2_data *data,
                const ftsens_ids_calibration_s *calibration, double adc[6]);

#ifdef __cplusplus
}
#endif


#endif /* H_FTSENS_CODELS */
